function [lGamma3x1term2,uGamma3x0term2,uGamma3x1term2,lGamma3x0term2] ...
    = Gamma3derivTerm2(Data,tuningParams,pmax,p1,p1deriv,x,y,h1,WsuppSize,pW)
% This function calculates the second term of the derivative of Gamma3
%
% Second term = 
% 1/N \sum_i \int (1,x,W_i)'gamma_deriv(S(x,W_i,tau,beta)) * 
% T(x,W_i,tau,beta,h1,kappa) d\tau
%
% Inputs:
%       input1 - data
%       input2 - tuning parameters
%       input3 - the max of (p1, 1-p1) for each observation
%       input4 - propensity score
%       input5 - derivative of propensity score
%       input6 - interpolation nodes for the quantile ceofficients, x
%       input7 - interpolation nodes for the quantile ceofficients, y
%       input8 - h1
%       input9 - size of the support of W. For continuous W, this is the
%       number of observations.
%       input10 - probability mass of W. Used only for discrete W.
% Outputs:
%       second term of the derivative of Gamma3
%
% Subfunctions: setGrid, quantileBounds, CQTEHDD
%
% Written by: Linqi Zhang
% October 2022

[~,cAll,cNum,tau] = setGrid(tuningParams,pmax);
nBoot = size(h1,2);

cqteHdd = nan(cNum,nBoot,4,length(tau));
lGamma2x0term2 = nan(cNum,nBoot,WsuppSize);
lGamma2x1term2 = nan(cNum,nBoot,WsuppSize);
uGamma2x0term2 = nan(cNum,nBoot,WsuppSize);
uGamma2x1term2 = nan(cNum,nBoot,WsuppSize);

for n = 1:WsuppSize
    suppW = Data.W(n,:);
    
    for i = 1:cNum
        
        c = cAll(i);
        qtlBounds = quantileBounds(suppW,tuningParams.eta,p1(n), ...
            tau,c,tuningParams.epsilon,x,y,0);

        for b = 1:nBoot % can be replaced by parfor depending on # of iterations of the outer loop

            cqteHdd(i,b,:,:) = CQTEHDD(qtlBounds,p1(n), ...
                p1deriv(n,:),tau,c,tuningParams.epsilon, ...
                tuningParams.kappa,h1(:,b));

        end
    end

    lGamma2x1term2(:,:,n) = mean(cqteHdd(:,:,1,:),4);
    uGamma2x0term2(:,:,n) = mean(cqteHdd(:,:,2,:),4);
    uGamma2x1term2(:,:,n) = mean(cqteHdd(:,:,3,:),4);
    lGamma2x0term2(:,:,n) = mean(cqteHdd(:,:,4,:),4);
    
end

if ~isempty(pW)
    lGamma3x0term2 = lGamma2x0term2(:,:,1)*pW(1)+lGamma2x0term2(:,:,2)*pW(2);
    uGamma3x0term2 = uGamma2x0term2(:,:,1)*pW(1)+uGamma2x0term2(:,:,2)*pW(2);
    lGamma3x1term2 = lGamma2x1term2(:,:,1)*pW(1)+lGamma2x1term2(:,:,2)*pW(2);
    uGamma3x1term2 = uGamma2x1term2(:,:,1)*pW(1)+uGamma2x1term2(:,:,2)*pW(2);
else
    lGamma3x0term2 = mean(lGamma2x0term2,3);
    uGamma3x0term2 = mean(uGamma2x0term2,3);
    lGamma3x1term2 = mean(lGamma2x1term2,3);
    uGamma3x1term2 = mean(uGamma2x1term2,3);
end