function computeCI(Data,DataBootstrap,tuningParams,alpha,...
    iSim,dgpChoice,tuningChoice)
% This function computes confidence sets on ATE and ATT bounds.
%
% Inputs:
%       input1 - data
%       input2 - bootstrap data
%       input3 - tuning parameters
%       input4 - significance level
%       input5 - (simulation) sample index
%       input6 - dgp choice index
%       input7 - tuning parameter choice index
%
% Outputs:
%       confidence sets
%
% Subfunctions: propensityScore, setGrid, quantileCoefNodes, 
%               Gamma3derivTerm2, Gamma3derivTerm1, Gamma3
%
% Written by: Linqi Zhang
% October 2022

%% 1. Import and generate datasets
%% 1.0 Load data

nObs = size(Data.X,1); 
nCov = size(Data.W,2);
nCov = nCov + 1; % Dimension of covariates including constant

%% 1.1 Set parameters

 
%% 1.2 Generate B bootstrap samples

nBoot = size(DataBootstrap.X, 3); % Number of bootstrap samples
tic
%% 2. Calculate propensity score(s)
% 
% beta:              the logit coefficient estimated with the original data
% beta_b:            the logit coefficient estimated with bootstrapped datasets
%
% Propensity scores: L(x, w'beta), where w is original covariates
%
% Derivative of 
% propensity scores: L_beta(x, w'beta)
%
% h1:                sqrt(N)*(beta_b - beta)
% 
if dgpChoice~=0
    
    [p1origin,p1derivOrigin,beta] = propensityScore(Data,[0;1]);
    pmax = max([p1origin,(1-p1origin)],[],2); 
    
else
    
    % P(X=1|W=Data.W) = L(1, Data.W'*beta)
    [p1origin,p1derivOrigin,beta] = propensityScore(Data);

    % Get max for each observation [Or use a single pmax for all?]
    pmax = max([p1origin,(1-p1origin)],[],2); 
    
end
    
[~,~,betaBoot] = propensityScore(DataBootstrap); 
h1 = sqrt(nObs) * (betaBoot - beta);

clear betaBoot;
toc
%% 3. Set the grid points
%
% Form matrix of c-dependence values
% Genrate tau grid points for numerical integration
%
[~,~,cNum,~] = setGrid(tuningParams,pmax);

%% 4. Steps required for asymptotic distribution of ATE bounds
%% 4.1 Approximate quantile coefficients: 
%
% Calculate interpolation nodes for the original sample
%
[x,y] = quantileCoefNodes(Data,tuningParams,nCov,1);
tic
%% 4.2.1 Derivative of Gamma3: second term
%
% Second term = 1/N \sum_i \int (1,x,W_i)'gamma_deriv(S(x,W_i,tau,beta)) *
%               T(x,W_i,tau,beta,h1,kappa) d\tau
%       

% c = parcluster;
% c.AdditionalProperties.WallTime='100:00:00'; 
% c.AdditionalProperties.MemUsage='3G'; 
% c.AdditionalProperties.ProcsPerNode=20;
% saveProfile(c)
% p = parpool(20)

if dgpChoice~=0
    pW         = [(nObs-sum(Data.W))/nObs;sum(Data.W)/nObs];
    Data.W     = [0;1];
    WsuppSize  = 2;
    
    saveSuffix = 'dgp%d_tuning%d_n%d_sim%d.mat';
    savePath   = 'output/fullMC';
else
    pW         = [];
    WsuppSize  = nObs;
    saveSuffix = 'dgp%d_tuning%d_n%d_sample%d.mat';
    savePath   = 'output/empirical';
end

[lGamma3x1term2,uGamma3x0term2,uGamma3x1term2,lGamma3x0term2] ...
        = Gamma3derivTerm2(Data,tuningParams, ...
          pmax,p1origin,p1derivOrigin,x,y,h1,WsuppSize,pW);

lAteHdd = lGamma3x1term2-uGamma3x0term2;
uAteHdd = uGamma3x1term2-lGamma3x0term2;

lTerm2  = lAteHdd;
uTerm2  = uAteHdd;

clear h1 lAteHdd uAteHdd p1derivOrigin;
% p.delete
% poolobj = gcp('nocreate')
% delete(poolobj);
toc
%% 4.2.2 Derivative of Gamma3: first term
%
% First term = 1/N \sum_i \int (1,x,W_i)'h2(S(x,W_i,tau,beta)) d\tau
% h2(S)      = sqrt(N)*(gamma_b^*(S) - gamma(S))
%

% c = parcluster;
% c.AdditionalProperties.WallTime='100:00:00'; 
% c.AdditionalProperties.MemUsage='3G';
% c.AdditionalProperties.ProcsPerNode=20;
% saveProfile(c)
% p = parpool(20)

[lGamma3x1term1,uGamma3x0term1,uGamma3x1term1,lGamma3x0term1, ...
       uGamma3x0,lGamma3x0,lAte,uAte] ...
       = Gamma3derivTerm1(Data,DataBootstrap,...
         tuningParams,pmax,p1origin,x,y,WsuppSize,pW);

lTerm1 = lGamma3x1term1-uGamma3x0term1;
uTerm1 = uGamma3x1term1-lGamma3x0term1;

saveFileName = sprintf(strcat(savePath,'/ate/ate_',saveSuffix),...
               dgpChoice,tuningChoice,nObs,iSim);
save(saveFileName,'lAte','uAte');

% p.delete
% poolobj = gcp('nocreate')
% delete(poolobj);
%% 4.2.3 Derivative of Gamma3
lGamma3deriv   = lTerm1+lTerm2; 
uGamma3deriv   = uTerm1+uTerm2;

% These are used to compute ATT CI
lGamma3x0deriv = lGamma3x0term1 + lGamma3x0term2;
uGamma3x0deriv = uGamma3x0term1 + uGamma3x0term2;

clear lCate uCate lAteBoot uAteBoot lTerm1 lTerm2 uTerm1 uTerm2 xb yb p1origin;

%% 4.3 Calculate the bootstrap empirical process

lGamma3x0bootEmp = nan(cNum,nBoot);
uGamma3x0bootEmp = nan(cNum,nBoot);

% c = parcluster;
% c.AdditionalProperties.WallTime='100:00:00'; 
% c.AdditionalProperties.MemUsage='3G'; 
% c.AdditionalProperties.ProcsPerNode=20;
% saveProfile(c)
% p = parpool(20)

parfor b = 1:nBoot
    
    suppW         = DataBootstrap.W(:,:,b);    
    if dgpChoice~=0
        pWboot    = [(nObs-sum(suppW))/nObs; sum(suppW)/nObs];
        suppW     = [0;1];
        WsuppSize = 2;
    else
        pWboot    = [];
        WsuppSize = nObs;
    end
    
    pIndex        = [ones(size(suppW,1),1),suppW] * beta(:,1);
    p1condBoot    = 1./(1+exp(-pIndex));
    pmax          = max([p1condBoot,(1-p1condBoot)],[],2); 
    
    [lGamma3x1bootEmp(:,b),uGamma3x0bootEmp(:,b), ...
        uGamma3x1bootEmp(:,b),lGamma3x0bootEmp(:,b)] ...
        = Gamma3(suppW,tuningParams,pmax,p1condBoot,x,y,WsuppSize,pWboot);
    
end

lAteBootEmp = lGamma3x1bootEmp-uGamma3x0bootEmp;
uAteBootEmp = uGamma3x1bootEmp-lGamma3x0bootEmp;

clear lCateBootEmp uCateBootEmp p1condBoot pIndex pmax qtlBounds x y;

lBootEmp    = sqrt(nObs) * (lAteBootEmp - lAte); 
uBootEmp    = sqrt(nObs) * (uAteBootEmp - uAte); 

lx0bootEmp  = sqrt(nObs) * (lGamma3x0bootEmp - lGamma3x0); 
ux0bootEmp  = sqrt(nObs) * (uGamma3x0bootEmp - uGamma3x0); 

clear lAteBootEmp uAteBootEmp;

lDist       = lGamma3deriv + lBootEmp;
uDist       = uGamma3deriv + uBootEmp;

clear lGamma3deriv lBootEmp uGamma3deriv uBootEmp;

% p.delete
% poolobj = gcp('nocreate')
% delete(poolobj);
%% 5. Construct confidence set

numList = [abs(lDist),abs(uDist)];
numList = sort(numList,2,'descend');
d       = nan(cNum,length(alpha));

for iAlpha = 1:length(alpha)
    a = alpha(iAlpha);
    for i = 1:cNum
        for j = 1:2*nBoot
            z = numList(i,j);
            flag = ((sum(lDist(i,:)<=z & uDist(i,:)>=-z)) ...
                >= (1-a)*nBoot);

            if flag==0
                break;
            end
        end

        d(i,iAlpha) = numList(i,j-1);
    end
end
         
CI = [lAte - d/sqrt(nObs),uAte + d/sqrt(nObs)];

saveFileName = sprintf(strcat(savePath,'/CI/CI_',saveSuffix),...
               dgpChoice,tuningChoice,nObs,iSim);
save(saveFileName,'CI');

%% 6. Compute the CI for the bounds on ATT

hatp1        = mean(Data.X);
hatp0        = 1-hatp1;

condMeanYx1  = mean(Data.X .* Data.Y)/hatp1;
condMeanYx0  = mean((1-Data.X).*Data.Y)/hatp0;

hatp1bootEmp = squeeze(sqrt(nObs)*(mean(DataBootstrap.X,1)-hatp1));
hatp0bootEmp = squeeze(sqrt(nObs)*(mean(DataBootstrap.X,1)-hatp0));
hatp1bootEmp = repmat(hatp1bootEmp',cNum,1);
hatp0bootEmp = repmat(hatp0bootEmp',cNum,1);
    
piece11      = squeeze(sqrt(nObs)*(mean(DataBootstrap.X .* DataBootstrap.Y,1)-...
               mean(Data.X .* Data.Y)))';
piece10      = squeeze(sqrt(nObs)*(mean((1-DataBootstrap.X) .* DataBootstrap.Y,1)-...
               mean((1-Data.X) .* Data.Y)))';

piece21      = condMeanYx1*hatp1bootEmp;
piece20      = condMeanYx0*hatp0bootEmp;

condMeanYx1bootEmp = (piece11-piece21)./hatp1;
condMeanYx0bootEmp = (piece10-piece20)./hatp0;

% Bounds on ATT

lAtt     = condMeanYx1 - (uGamma3x0-hatp0*condMeanYx0)/hatp1;
uAtt     = condMeanYx1 - (lGamma3x0-hatp0*condMeanYx0)/hatp1;

% Inference

lDistAtt = condMeanYx1bootEmp - (lGamma3x0deriv+lx0bootEmp)/hatp1 +...
           (hatp0/hatp1)*condMeanYx0bootEmp +...
           (condMeanYx0/hatp1)*hatp0bootEmp +...
           (lGamma3x0 - condMeanYx0*hatp0)/hatp1^2 .* hatp1bootEmp;

uDistAtt = condMeanYx1bootEmp - (uGamma3x0deriv+ux0bootEmp)/hatp1 +...
           (hatp0/hatp1)*condMeanYx0bootEmp +...
           (condMeanYx0/hatp1)*hatp0bootEmp +...
           (uGamma3x0 - condMeanYx0*hatp0)/hatp1^2 .* hatp1bootEmp;

numList  = [abs(lDistAtt),abs(uDistAtt)];
numList  = sort(numList,2,'descend');
d        = nan(cNum,length(alpha));

for iAlpha = 1:length(alpha)
    a = alpha(iAlpha);
    for i = 1:cNum
        for j = 1:2*nBoot
            z = numList(i,j);
            flag = ((sum(lDistAtt(i,:)<=z & uDistAtt(i,:)>=-z)) ...
                >= (1-a)*nBoot);

            if flag==0
                break;
            end
        end

        d(i,iAlpha) = numList(i,j-1);
    end
end

CI = [lAtt - d/sqrt(nObs),uAtt + d/sqrt(nObs)];

saveFileName = sprintf(strcat(savePath,'/att/att_',saveSuffix),...
               dgpChoice,tuningChoice,nObs,iSim);
save(saveFileName,'lAtt','uAtt');

saveFileName = sprintf(strcat(savePath,'/CI/CIatt_',saveSuffix),...
               dgpChoice,tuningChoice,nObs,iSim);
save(saveFileName,'CI');
 
end