function qtlBounds = quantileBounds(w,eta,p1,tau,c,epsilon,xInterpolate,...
    yInterpolate,noDeriv)
% This function calculates the quantile function
%
% This is part of CQTEHDD: (1,x,w)'*\gamma'(S2(x,w,tau,beta))
% If noderiv == 1, then it calculates (1,x,w)'*\gamma(S2(x,w,tau,beta))
% If noderiv == 2, then it calculates (1,x,w,x*w)'*\gamma(S2(x,w,tau,beta))
% It is computed separately because this term doesn't vary across
% bootstrapped samples b. We only need to compute it once for each c and w.
%
% 1. lower bound, x = 1
% 2. upper bound, x = 0
% 3. upper bound, x = 1
% 4. lower bound, x = 0
%
% k is the number of covariates excluding constant
% l is the dimension of tau
% n is the number of rows of w (restricting to n=1 for now)
% xw is a n * (k+2) matrix
% p1 is a scalar, the propensity score evaluated at a given w, P(X=1|W=w) 
% coef1/coef2/coef_deriv are (k+2) * l matrices
% OUTPUT qtlBounds is a 4 * l matrix
% Note that if n=1, we'll need to loop over observations when calculating
% ATE. 
%
% Written by: Linqi Zhang
% October 2022

nSuppW = size(w,1);
qtlBounds = nan(4,length(tau));

if noDeriv == 1
    for j = 1:4
        if mod(j,2) == 0
            x = 0;
            xw = [ones(nSuppW,1),zeros(nSuppW,1),w];            
        else 
            xw = [ones(nSuppW,1),ones(nSuppW,1),w];
            x = 1;
        end

        if (j == 2) || (j==3)
            qtl = S2(p1,x,tau,c,epsilon);
        else 
            qtl = S4(p1,x,tau,c,epsilon);
        end

        coef = pchip(xInterpolate,yInterpolate,qtl);
        qtlBounds(j,:) = xw * coef;

    end   
elseif noDeriv == 2
    for j = 1:4
        if mod(j,2) == 0
            x = 0;
            xw = [ones(nSuppW,1),zeros(nSuppW,1),w,w.*x];            
        else 
            x = 1;
            xw = [ones(nSuppW,1),ones(nSuppW,1),w,w.*x];            
        end

        if (j == 2) || (j==3)
            qtl = S2(p1,x,tau,c,epsilon);
        else 
            qtl = S4(p1,x,tau,c,epsilon);
        end

        coef = pchip(xInterpolate,yInterpolate,qtl);
        qtlBounds(j,:) = xw * coef;
    end       
elseif noDeriv == 0
    for j = 1:4
        if mod(j,2) == 0
            xw = [ones(nSuppW,1),zeros(nSuppW,1),w];
            x = 0;
        else 
            xw = [ones(nSuppW,1),ones(nSuppW,1),w];
            x = 1;
        end

        if (j == 2) || (j==3)
            qtl = S2(p1,x,tau,c,epsilon);
        else 
            qtl = S4(p1,x,tau,c,epsilon);
        end

        coef1 = pchip(xInterpolate,yInterpolate, qtl+eta);
        coef2 = pchip(xInterpolate,yInterpolate, qtl-eta);

        coefDeriv = (coef1 - coef2)/(2*eta);
        qtlBounds(j,:) = xw * coefDeriv;
    end
end
end