function [Gamma1upper,Gamma1lower] = trueGamma1(params,tau,c,tuningParams,epsN,...
    betaZ1,xb,yb)
% This function calculates the true values of Gamma1 bounds and its 
% derivative.
%
% nargin == 4, returns the true upper bound of Gamma1
% nargin == 5, returns the numerical derivative of the true upper bound of 
% Gamma1 evaluated at h1 = beta, h2 = gamma.
% nargin == 8, returns the numerical derivative of the true upper bound of 
% Gamma1 evaluated at h1 = sqrt(nObs)*(betaZ1-beta), 
% h2 = sqrt(nObs)*(gammaZ1-gamma)
%
% Note that, technically, it returns the trimmed population quantile
% bounds, which is the population analog of the bound functions defined in
% equations (6) and (7) in MPZ(2022).
%
% Written by: Linqi Zhang
% October 2022

if ismember(params.dgpChoice, [1 5 6 7 8])
    
    XsuppSize = 2;
    WsuppSize = 2;
    Gamma1upper = nan(length(tau),XsuppSize,WsuppSize);
    Gamma1lower = nan(length(tau),XsuppSize,WsuppSize);
    
    if nargin == 4
        for xi=1:XsuppSize
            x = xi - 1;

            for wi=1:WsuppSize
                w = wi - 1;
                p1 = w*params.p1w1true + (1-w)*params.p1w0true;
                S2Value = S2(p1, x, tau, c, tuningParams.epsilon);
                Gamma1upper(:,xi,wi) = trueQuantileFunction_MC(params,S2Value,x,w);
 
                S4Value = S4(p1, x, tau, c, tuningParams.epsilon);
                Gamma1lower(:,xi,wi) = trueQuantileFunction_MC(params,S4Value,x,w);
            end
        end
    
    elseif nargin == 5         
        h1 = params.betaTrue;
        for xi=1:XsuppSize
            x = xi - 1;

            for wi=1:WsuppSize
                w = wi - 1;
                p1epsh = 1./(1+exp(-[1,w]*(params.betaTrue+epsN*h1)));
                p1 = w*params.p1w1true + (1-w)*params.p1w0true;
                S2ValueEps = S2(p1epsh, x, tau, c, tuningParams.epsilon);
                S2Value = S2(p1, x, tau, c, tuningParams.epsilon);
                Gamma1upper(:,xi,wi) = ...
                    ((1+epsN)*trueQuantileFunction_MC(params,S2ValueEps,x,w) - ...
                    trueQuantileFunction_MC(params,S2Value,x,w))/epsN;
                
                S4ValueEps = S4(p1epsh, x, tau, c, tuningParams.epsilon);
                S4Value = S4(p1, x, tau, c, tuningParams.epsilon);
                Gamma1lower(:,xi,wi) = ...
                    ((1+epsN)*trueQuantileFunction_MC(params,S4ValueEps,x,w) - ...
                    trueQuantileFunction_MC(params,S4Value,x,w))/epsN;
            end
        end
    else
        h1 = sqrt(params.nObs)*(betaZ1-params.betaTrue);
        Gamma1upperPiece1 = nan(length(tau),XsuppSize,WsuppSize);
        Gamma1lowerPiece1 = nan(length(tau),XsuppSize,WsuppSize);
        
        for wi=1:WsuppSize
            w = wi - 1;
            p1epsh = 1./(1+exp(-[1,w]*(params.betaTrue+epsN*h1)));
            p1 = w*params.p1w1true + (1-w)*params.p1w0true;
            Gamma1upperTemp = quantileBounds(w,[],p1epsh,tau,c, ...
                tuningParams.epsilon,xb,yb,1);
            Gamma1lowerTemp = quantileBounds(w,[],p1epsh,tau,c, ...
                tuningParams.epsilon,xb,yb,1);
            for xi=1:XsuppSize
                x = xi - 1;

                S2ValueEps = S2(p1epsh,x,tau,c,tuningParams.epsilon);
                S2Value = S2(p1,x,tau,c,tuningParams.epsilon);
                
                Gamma1upperPiece1(:,xi,wi) = ...
                    (1-epsN*sqrt(params.nObs))*...
                    trueQuantileFunction_MC(params,S2ValueEps,x,w)+ ...
                    epsN*sqrt(params.nObs)*Gamma1upperTemp(2+x,:);
                
                Gamma1upper(:,xi,wi) = (Gamma1upperPiece1(:,xi,wi)' - ...
                    trueQuantileFunction_MC(params,S2Value,x,w))/epsN;
                
                S4ValueEps = S4(p1epsh,x,tau,c,tuningParams.epsilon);
                S4Value = S4(p1,x,tau,c,tuningParams.epsilon);
                
                Gamma1lowerPiece1(:,xi,wi) = ...
                    (1-epsN*sqrt(params.nObs))*...
                    trueQuantileFunction_MC(params,S4ValueEps,x,w)+ ...
                    epsN*sqrt(params.nObs)*Gamma1lowerTemp(2+x,:);
                
                Gamma1lower(:,xi,wi) = (Gamma1lowerPiece1(:,xi,wi)' - ...
                    trueQuantileFunction_MC(params,S4Value,x,w))/epsN;
                
            end
            
        end        
    end
    
elseif ismember(params.dgpChoice, [10 11 12])
    
    XsuppSize = 2;
    nWsupp = tuningParams.RIntgrid;
    Gamma1upper = nan(length(tau),XsuppSize,nWsupp);
    Gamma1lower = nan(length(tau),XsuppSize,nWsupp);
    load(sprintf('output/dgp/randomDraws/normaldraw_R%d',tuningParams.RIntgrid),'Wsupp');
    
    rescaleW = 2*pi*normcdf(Wsupp)-pi;
    sinW = sin([rescaleW 2*rescaleW 3*rescaleW]);
    cosW = cos([rescaleW 2*rescaleW 3*rescaleW]);
    kW = params.lambda0 + sinW*params.lambdaS + cosW*params.lambdaC;
    p1vec = 1./(1+exp(-kW));
    
    if nargin == 4
        for xi=1:XsuppSize
            x = xi - 1;
            
            parfor wi=1:nWsupp
                
                w = Wsupp(wi);
                p1 = p1vec(wi);
                S2Value = S2(p1,x,tau,c,tuningParams.epsilon);
                Gamma1upper(:,xi,wi) = trueQuantileFunction_MC(params,S2Value,x,w);
                
                S4Value = S4(p1,x,tau,c,tuningParams.epsilon);
                Gamma1lower(:,xi,wi) = trueQuantileFunction_MC(params,S4Value,x,w);
                
            end
        end        
    
    elseif nargin > 4
        error('Too many arguments. For dgpChoice >= 10, only allow for calculating the true value of bound function.')
    end

elseif  ismember(params.dgpChoice, [13 14 15])
    
    XsuppSize = 2;
    nWsupp = tuningParams.RIntgrid;
    Gamma1upper = nan(length(tau),XsuppSize,nWsupp);
    Gamma1lower = nan(length(tau),XsuppSize,nWsupp);
    load(sprintf('output/dgp/randomDraws/normaldraw_R%d',tuningParams.RIntgrid),'Wsupp');
    
    kW = params.lambda0 + params.lambda1*Wsupp + ...
        params.lambda2*Wsupp.^2 + params.lambda3*Wsupp.^3;
    p1vec = 1./(1+exp(-kW));
    
    if nargin == 4
        for xi=1:XsuppSize
            x = xi - 1;
            
            parfor wi=1:nWsupp
                
                w = Wsupp(wi);
                p1 = p1vec(wi);
                S2Value = S2(p1,x,tau,c,tuningParams.epsilon);
                Gamma1upper(:,xi,wi) = trueQuantileFunction_MC(params,S2Value,x,w);
                
                S4Value = S4(p1,x,tau,c,tuningParams.epsilon);
                Gamma1lower(:,xi,wi) = trueQuantileFunction_MC(params,S4Value,x,w);
                
            end
        end        
    
    elseif nargin > 4
        error('Too many arguments. For dgpChoice >= 10, only allow for calculating the true value of bound function.')
    end    
end
end