// This code implements the sensitivity analysis 
// in Masten and Poirier (2018) and Masten, Poirier, and Zhang (2020)
//
// This analysis uses the classical LaLonde (1986) data, as reconstructed by
// Dehejia and Wahba. We donwloaded all of our data on May 29, 2018 from 
// Dehejia's site at the NBER:
//
// http://users.nber.org/~rdehejia/data/nswdata2.html
//
// We perform a similar analysis as Imbens (2003), who looked at three different 
// versions of the LaLonde datasets. In our main results, we presents 
// sensitivity analysis for two of the three datasets he used.

// To install the companion Stata module
ssc install tesensitivity
capture log close
log using "replication", smcl replace

// Experimental Dataset: Dehejia-Wahba sample.
// This is the baseline sample Imbens (2003) uses.
// Unlike the previous dataset, it has data on 1974 earnings too (re74)
//
// We use the same set of covariates as Imbens (2003):
// Married, age, indicators for black and hispanic, education, earnings in 1974
// and 1975, and indicators for positive earnings in 1974 and 1975
use DehejiaData/nsw_dw.dta, clear
compress
do prepLaLondeData.do
save DehejiaData/nsw_dw_temp.dta, replace

// Observational Dataset:
use DehejiaData/nsw_dw.dta, clear
drop if treat == 0
append using DehejiaData/psid_controls.dta
compress
drop if re74 > 5000
drop if re75 > 5000
do prepLaLondeData.do
append using DehejiaData/nsw_dw_temp.dta, gen(sample1)
gen sample3 = 1-sample1
erase DehejiaData/nsw_dw_temp.dta
save DehejiaData/lalonde1986, replace


// Define variable lists
local outcome "re78"
local treatment "treat"
local controls "married age black hispanic education re74 re75 re74pos re75pos"

// Table S2 Get summary stats
get_summary_stats `controls' if sample1
matrix d1sumStatsTable = r(sumStatsTable)
get_summary_stats `controls' if sample3
matrix d3sumStatsTable = r(sumStatsTable)
matrix AllSumStatsTable = (d1sumStatsTable, d3sumStatsTable[1...,1])

// Table 1 Baseline ATE and ATT estimates, experimental dataset
teffects ipw (`outcome') (`treatment' `controls') if sample1
teffects ipw (`outcome') (`treatment' `controls') if sample1, atet

// Table 1 Baseline ATE and ATT estimates, observational dataset
teffects ipw (`outcome') (`treatment' `controls') if sample3
teffects ipw (`outcome') (`treatment' `controls') if sample3, atet


// SENSITIVITY ANALYSIS, EXPERIMENTAL DATA //

tesensitivity cpi (`outcome' `controls') (`treatment' `controls') if sample1, ate 

// Table 2 Leave-Out-Variable k propensity score analysis
tesensitivity cscale

// Figure 2
tesensitivity cscale hispanic, density 
graph export images/propScoreVar_hispanic.eps, replace

// Table 3 Leave-Out-Variable k ATE analysis
leave_out_ate `outcome' `treatment' `controls' if sample1
matrix colname ATEdiffRatio = "ExperimentalDataset"
matrix allATEdiffs = ATEdiffRatio
matrix rowname baselineEsts = "ExperimentalDataset"
matrix BaselineEsts = baselineEsts

// SENSITIVITY ANALYSIS, OBSERVATIONAL DATA //

tesensitivity cpi (`outcome' `controls') (`treatment' `controls') if sample3, ate 

// Table 4 Leave-Out-Variable k propensity score analysis
tesensitivity cscale

// Table 3 Leave-Out-Variable k ATE analysis
leave_out_ate `outcome' `treatment' `controls' if sample3
matrix colname ATEdiffRatio = "ObservationalDataset"
matrix allATEdiffs = allATEdiffs,ATEdiffRatio
matrix rowname baselineEsts = "ObservationalDataset"
matrix BaselineEsts = (BaselineEsts\baselineEsts)
matrixsort allATEdiffs 1

outtable using baselineEsts, mat(BaselineEsts) f(%9.0f) nobox ///
    caption("Baseline treatment effect estimates (in 1978 dollars). Standard errors in parentheses.") ///
    clabel(baselineEsts) center replace
outtable using ATEdiffs, mat(allATEdiffs) f(%9.2f) nobox label ///
    caption("Effect of omitting a single variable on ATE point estimates (as a percentage of the baseline estimate).") ///
    clabel(tableATEdiffs) center replace
outtable using summaryStats, mat(AllSumStatsTable) f(%9.2f) nobox ///
    caption("Summary statistics.") ///
    clabel(summaryStats) center replace
